#!/usr/bin/env python
"""
Simple script to parse cassandra-stress' transaction results
and reformat them as JSON for sending back to juju
"""
import sys
import subprocess
import json
from charmhelpers.contrib.benchmark import Benchmark


def action_set(key, val):
    action_cmd = ['action-set']
    if isinstance(val, dict):
        for k, v in val.iteritems():
            action_set('%s.%s' % (key, k), v)
        return

    action_cmd.append('%s=%s' % (key, val))
    subprocess.check_call(action_cmd)


def parse_stress_output():
    """
    Parse the output from cassandra-stress and set the action results
    """

    a = sys.stdin.read()
    action_set("meta.raw", a)

    l = [" ".join(s.split()) for s in a.split('\n')]
    r = l[l.index('Results:')+1:l.index('END')]

    keys = {}
    for q in r:
        for (k, v) in [(q.split(' : ')[0].lower(), q.split(' : ')[1].split(' [')[0])]:
            k = k.replace('(ms)', '').replace('(s)', '').strip().replace(' ', '-')
            keys[k] = v

    # op-rate:1413
    # partition-rate:1413
    # row-rate:1413
    # latency-mean:143.0
    # latency-median:92.5
    # latency-95th-percentile:394.9
    # latency-99th-percentile:1179.3
    # latency-99.9th-percentile:1266.0
    # latency-max:1308.9
    # total-partitions:10000
    # total-errors:0
    # total-gc-count:2
    # total-gc-mb:44
    # total-gc-time-:0
    # avg-gc-time:55
    # stdev-gc-time:40
    # total-operation-time:00:00:07

    action_set(
        "results.op-rate",
        {'value': keys['op-rate'], 'units': 'op/s'}
    )

    action_set(
        "results.partiton-rate",
        {'value': keys['partition-rate'], 'units': 'pk/s'}
    )

    action_set(
        "results.row-rate",
        {'value': keys['row-rate'], 'units': 'row/s'}
    )

    action_set(
        "results.latency-mean",
        {'value': keys['latency-mean'], 'units': ''}
    )

    action_set(
        "results.latency-median",
        {'value': keys['latency-median'], 'units': ''}
    )

    action_set(
        "results.latency-95th-percentile",
        {'value': keys['latency-95th-percentile'], 'units': ''}
    )

    action_set(
        "results.latency-99th-percentile",
        {'value': keys['latency-99th-percentile'], 'units': ''}
    )

    action_set(
        "results.latency-max",
        {'value': keys['latency-max'], 'units': ''}
    )

    action_set(
        "results.total-partitions",
        {'value': keys['total-partitions'], 'units': ''}
    )

    action_set(
        "results.total-errors",
        {'value': keys['total-errors'], 'units': ''}
    )

    action_set(
        "results.total-gc-count",
        {'value': keys['total-gc-count'], 'units': ''}
    )

    action_set(
        "results.total-gc-mb",
        {'value': keys['total-gc-mb'], 'units': 'MB'}
    )

    action_set(
        "results.total-gc-time",
        {'value': keys['total-gc-time'], 'units': ''}
    )

    action_set(
        "results.avg-gc-time",
        {'value': keys['avg-gc-time'], 'units': ''}
    )

    action_set(
        "results.stdev-gc-time",
        {'value': keys['stdev-gc-time'], 'units': ''}
    )

    action_set(
        "results.total-operation-time",
        {'value': keys['total-operation-time'], 'units': ''}
    )

    Benchmark.set_composite_score(
        keys['op-rate'],
        'ops/sec',
        'desc'
    )


if __name__ == "__main__":
    parse_stress_output()
